import ssl
import ftplib
import io
import datetime


class Ftps(ftplib.FTP_TLS):
    """FTPS subclass that automatically wraps sockets in SSL to support implicit FTPS."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._sock = None

    @property
    def sock(self):
        """Return the socket."""
        return self._sock

    @sock.setter
    def sock(self, value):
        """When modifying the socket, ensure that it is ssl wrapped."""
        if value is not None and not isinstance(value, ssl.SSLSocket):
            value = self.context.wrap_socket(value)
        self._sock = value

    def get_file(self, path):
        """Read and return data from file which is specified 'path' """
        temp_data = io.BytesIO()
        self.retrbinary("RETR " + path, temp_data.write)
        temp_data.seek(0)
        return temp_data

    def list_directory(self, dirName):
        dirs = []
        self.dir(dirName, dirs.append)
        files = []
        for dirr in dirs:
            line = dirr.lstrip().split()
            datetmp = " ".join(line[5 : len(line) - 1])  # noqa: E203
            month, day, year = datetmp.split(" ")
            time = None
            now = datetime.datetime.now()
            if year.find(":") > -1:
                time = year
                year = now.year
            else:
                time = "00:00"
            files.append(
                {
                    "permission": line[0],
                    "links": line[1],
                    "owner": line[2],
                    "group": line[3],
                    "size": line[4],
                    "month": month,
                    "day": day,
                    "year": year,
                    "time": time,
                    "name": line[-1],
                }
            )
        return files

    def help(self):
        print(f"=== {type(self).__name__} === \n")
        print(".get_file(path) => get file from ftp path\n")
        print(".list_directory(path) => do ls in ftp in path\n")



import requests
import os


class Pdf:
    """ PDF generator lib"""

    __url_pdf_api = os.environ.get("CS_API_PDF", "http://cashstory-pdf:9000")

    def generate(self, url, filename="generated.pdf"):
        """ generate pdf from html url with optional filename"""
        r = requests.get(
            f"{self.__url_pdf_api}/api/render/?url={url}&pdf.width=20.5cm&pdf.height=36.5cm"
        )
        open(filename, "wb").write(r.content)



import requests
from datetime import date
import os


class HealthCheck:
    healthUrl = None

    def __init__(self, mode="prod", healthUrl=None):
        """ Create health driver mode='prod', healthUrl=None"""
        print("HealthCheck init in " + mode + " mode")
        self.healthUrl = healthUrl if healthUrl else os.environ.get("HC_API", healthUrl)
        self.mode = mode

    def start(self, healthkey):
        """ send start to healthcheck (healthkey)"""
        if self.mode == "prod":
            try:
                requests.get(f"{self.healthUrl}{healthkey}/start")
                return f"Start ==> send to {self.healthUrl}{healthkey}, {date.today()}"
            except requests.exceptions.RequestException:
                return f"Error ==> cannot get health server {self.healthUrl}{healthkey}, {date.today()}"

    def done(self, healthkey):
        """ send done to healthcheck (healthkey)"""
        if self.mode == "prod":
            try:
                requests.get(f"{self.healthUrl}{healthkey}")
                return f"Done ==> send to {self.healthUrl}{healthkey}, {date.today()}"
            except requests.exceptions.RequestException:
                return f"Error ==> cannot get health server {self.healthUrl}{healthkey}, {date.today()}"

    def fail(self, healthkey):
        """ send fail to healthcheck (healthkey)"""
        if self.mode == "prod":
            try:
                requests.get(f"{self.healthUrl}{healthkey}/fail")
                return f"Fail ==> send to {self.healthUrl}{healthkey}, {date.today()}"
            except requests.exceptions.RequestException:
                return f"Error ==> cannot get health server {self.healthUrl}{healthkey}, {date.today()}"

    def checkUp(self, url, healthkey, auth=None, verify=True):
        """ check if url is reachable (url, healthkey, auth=None, verify=True)"""
        self.start(healthkey)
        try:
            r = requests.get(url, auth=auth, verify=verify)
            if r.status_code == 200:
                self.done(healthkey)
                return f"{url} is heathy send to {self.healthUrl}{healthkey} {date.today()}"
            else:
                self.fail(healthkey)
                return f"===>Fail {url} send to {self.healthUrl}{healthkey}, {date.today()}"
        except requests.exceptions.RequestException:
            return f"Error ==> cannot get health server {self.healthUrl}{healthkey}, {date.today()}"



import ftplib
import pysftp
import io


class Ftp:
    """FTP/SFTP Connector class for cashstory\nSample Credentials file :{
        "url": "",
        "protocole": "ftp or sftp",
        "port": ,
        "type": "explicit TLS",
        "auth_method": "Passif or Actif",
        "username": "",
        "password": ""
    }"""

    def __enter__(self):
        return self

    def get_file(self, path):
        """Read and return data from file which is specified 'path' """
        if self.conprotocol == "ftp":
            temp_data = io.BytesIO()
            self.ftp.retrbinary("RETR " + path, temp_data.write)
            temp_data.seek(0)
            return temp_data
        elif self.conprotocol == "sftp":
            with self.ftp.open(path) as f:
                return f

    def list_directory(self, path):
        files = []
        try:
            files = self.ftp.nlst()
        except ftplib.error_perm:
            print("Error read this directory")
        return files

    def __init__(self, credentials):
        if credentials["protocole"] == "ftp":
            ftp = (
                ftplib.FTP_TLS(credentials["url"])
                if credentials["type"] == "explicit TLS"
                else ftplib.FTP(credentials["url"])
            )
            ftp.connect(credentials["url"], credentials["port"])
            ftp.set_pasv(True) if credentials[
                "auth_method"
            ] == "Passif" else ftp.set_pasv(False)
            ftp.login(credentials["username"], credentials["password"])
            self.ftp = ftp
        elif credentials["protocole"] == "sftp":
            self.ftp = pysftp.Connection(
                host=credentials["url"],
                username=credentials["username"],
                password=credentials["password"],
                port=credentials["port"],
            )
        self.conprotocol = credentials["protocole"]

    def __exit__(self, *krgs):
        self.ftp.close()

    def help(self):
        print(f"=== {type(self).__name__} === \n")
        print(".get_file(path) => get file from ftp path\n")
        print(".list_directory(path) => do ls in ftp in path\n")



from .darkknight import DarkKnight
from .ftp import Ftp
from .ftps import Ftps
from .geolocator import GeoLocator
from .git import Git
from .google_spreadsheet import GoogleSpreadsheet
from .healthcheck import HealthCheck
from .email import Email
from .mongo import Mongo
from .pdf import Pdf
from .plot import Plot
from .prediction import Prediction
from .sentiment_analysis import SentimentAnalysis
from .toucan import Toucan

__version__ = "0.3.1"

darkknight = DarkKnight
ftp = Ftp
git = Git
ftps = Ftps
geoLocator = GeoLocator
google_spreadsheet = GoogleSpreadsheet
healthCheck = HealthCheck
email = Email
mongo = Mongo
pdf = Pdf
plot = Plot
prediction = Prediction
sentiment_analysis = SentimentAnalysis
toucan = Toucan


def help():
    print("=== Drivers === \n")
    print("drivers.darkknight() => Init the driver to connect to our backend \n")
    print("drivers.ftp() => Init the driver to connect to ftp\n")
    print("drivers.ftps() => Init the driver to connect to ftps\n")
    print("drivers.git() => Init the driver to connect to git\n")
    print("drivers.healthcheck() => Init the driver to connect to healthcheck\n")
    print(
        "drivers.google_spreadsheet() => Init the driver to connect to GoogleSpreadsheet\n"
    )
    print("drivers.mailer() => Init the driver to send email\n")
    print("drivers.mongo() => Get the Mongo driver\n")
    print("drivers.pdf() => Get the pdf generator driver\n")
    print("drivers.sentiment() => Get the sentiment driver\n")
    print("drivers.ml() => Get the machine learning driver\n")
    print("drivers.geo() => Get the GeoLocator driver\n")
    print("drivers.plot() => Get the plot driver\n")
    print("drivers.pdf() => Get the pdf driver\n")
